shader{
	code= "
	
		#define EARTH_RADIUS					6350.0	//Km
		#define ATMOSPHERE_HEIGHT				120.0	//Km
		#define NUM_STEPS						30
	
		uniform float							rayleighParticlesConcentration;
		uniform float							mieParticlesConcentration;
		uniform float							miePhaseFunctionG;
		uniform vec3							rayleighColorScatteringCoefficients;
		uniform vec3							mieColorScatteringCoefficients;		
		
		vec3 getPositionRelativeToEarthCenter(vec3 position){
			//Get the position above the earth surface. Note that the
			//position must be translated from meters to kilometers.
			return vec3(0.0, EARTH_RADIUS, 0.0)+ position/1000.0;
		}

		vec3 getAtmosfereIntersection(vec3 point, vec3 vector){
			float dvv= dot(vector, vector);
			float dpp= dot(point, point);
			float dpv= dot(point, vector);
			
			float radius= EARTH_RADIUS+ ATMOSPHERE_HEIGHT;
			float d= (-dpv+ sqrt(dpv*dpv- (dvv*dvv)*(dpp- radius*radius)))/dvv;
			
			return point+ vector*d;
		}

		float getAirDensity(vec3 point){
			float height= length(point)- EARTH_RADIUS;				
			return rayleighParticlesConcentration*exp(-height/ATMOSPHERE_HEIGHT);
		}

		float getAerosolDensity(vec3 point){
			float height= length(point)- EARTH_RADIUS;				
			return mieParticlesConcentration*exp(-height/ATMOSPHERE_HEIGHT);
		}
		
		
		float getRayleighPhase(vec3 viewVector, vec3 sunLightVector){
			float angle= dot(viewVector, sunLightVector);
			return (2.0+ 0.5*angle*angle);
		}

		float getMiePhase(vec3 viewVector, vec3 sunLightVector){
			float angle= dot(viewVector, sunLightVector);
			return (1.0- (miePhaseFunctionG*miePhaseFunctionG))/pow(1.0- 2.0*miePhaseFunctionG*angle+ miePhaseFunctionG*miePhaseFunctionG, 1.5);
		}
		
		//This function calculates an attenuatiuon factor that is used to smoothly
		//simulate the blocking of the sun rays by the earth it self. This is fundamental
		//to achieve acceptable results in dusk and night situations.
		float getOcclusion(vec3 viewPosition, vec3 viewVector){

			vec3 closestPoint= viewPosition+ viewVector*max(dot(-viewPosition, viewVector), 0.0);

			return clamp(pow(length(closestPoint)/(EARTH_RADIUS), 100.0), 0.0, 1.0);
		}

		//This function returns the scattering for a point in the atmosfere.
		vec3 getScattering(vec3 viewVectorPoint, vec3 viewVector, vec3 sunLightVector, float viewVectorRayleighOpticalDepth, float viewVectorMieOpticalDepth, vec3 rayleighMultiplier, vec3 mieMultiplier, float stepSize){
			
			//Calculate the light vector optical depth.
			vec3 atmosfereEntryPoint= getAtmosfereIntersection(viewVectorPoint, -sunLightVector);
			float lightVectorStepSize= length(atmosfereEntryPoint- viewVectorPoint)/(NUM_STEPS- 1);
			vec3 lightVectorPoint= viewVectorPoint;
			float lightVectorRayleighOpticalDepth= 0.0;
			float lightVectorMieOpticalDepth= 0.0;
			
			for(int j= 0; j< NUM_STEPS; ++j){
				lightVectorRayleighOpticalDepth+= getAirDensity(lightVectorPoint)*lightVectorStepSize;
				lightVectorMieOpticalDepth+= getAerosolDensity(lightVectorPoint)*lightVectorStepSize;
				lightVectorPoint+= sunLightVector*lightVectorStepSize;
			}
			
			//Calculate in-scatter.
			vec3 inScatterRayleigh= rayleighColorScatteringCoefficients*getRayleighPhase(viewVector, -sunLightVector)*getAirDensity(viewVectorPoint);
			vec3 inScatterMie= mieColorScatteringCoefficients*getMiePhase(viewVector, -sunLightVector)*getAerosolDensity(viewVectorPoint);
			
			//Calculate accumulated optical depth.
			vec3 opticalDepthRayleigh= rayleighColorScatteringCoefficients*(viewVectorRayleighOpticalDepth+ lightVectorRayleighOpticalDepth);
			vec3 opticalDepthMie= mieColorScatteringCoefficients*(viewVectorMieOpticalDepth+ lightVectorMieOpticalDepth);
			
			//Calculate the occlusion caused by the earth blocking the sun light.
			float occlusion= getOcclusion(viewVectorPoint, -sunLightVector);
			
			return (inScatterRayleigh*rayleighMultiplier+ inScatterMie*mieMultiplier)*exp(-opticalDepthRayleigh-opticalDepthMie)*stepSize*occlusion;
		}

		vec3 calculateAtmosphericScattering(vec3 position, vec3 cameraPosition, vec3 sunLightVector, vec3 rayleighMultiplier, vec3 mieMultiplier){
			vec3 viewVector= normalize(position- cameraPosition);			
			
			vec3 color= vec3(0, 0, 0);		
			
			float viewVectorStepSize= length(position- cameraPosition)/(NUM_STEPS- 1);
			float viewVectorRayleighOpticalDepth= 0.0;
			float viewVectorMieOpticalDepth= 0.0;
			vec3 viewPoint= cameraPosition;
			
			for(int i= 0; i< NUM_STEPS; ++i){
				viewVectorRayleighOpticalDepth+= getAirDensity(viewPoint)*viewVectorStepSize;
				viewVectorMieOpticalDepth+= getAerosolDensity(viewPoint)*viewVectorStepSize;
				
				color+= getScattering(viewPoint, viewVector, sunLightVector, viewVectorRayleighOpticalDepth, viewVectorMieOpticalDepth, rayleighMultiplier, mieMultiplier, viewVectorStepSize);
				viewPoint+= viewVector*viewVectorStepSize;
			}
			
			return color;
		}

		//Calculates the atmospheric scattering of a point in the sky. The fragment position that this fucntion
		//receives as a parameter must be previously normalized because it will be treated as a vector.
		vec3 getSkyAtmosphericScattering(vec3 fragmentPosition, vec3 cameraPosition, vec3 sunLightVector){
			
			//Apply some bias to the position to avoid numerical issues.
			fragmentPosition *= 0.9999;

			//Only the camera's y coordinate is important for this calculation.
			vec3 realCameraPosition= getPositionRelativeToEarthCenter(vec3(0.0, cameraPosition[1], 0.0));
			vec3 realPositionInAtmosfere= getAtmosfereIntersection(realCameraPosition, fragmentPosition);

			//Calculate the sun light scattering.
			vec3 color= calculateAtmosphericScattering(realPositionInAtmosfere, realCameraPosition, sunLightVector, 3.0, 800.0);

			//Calculate the moon light scattering.
			color+= calculateAtmosphericScattering(realPositionInAtmosfere, realCameraPosition, normalize(vec3(-1, -1, -1)), 0.0033, 30.0);
			
			return color;
		}
		
		vec3 calculateSunLightColor(vec3 sunLightVector){		
		
			//Define the original sun light color before it reaches the atmosphere.
			const vec3 sunColor= vec3(1.8);
		
			vec3 cameraPosition= getPositionRelativeToEarthCenter(vec3(0.0, 0.0, 0.0));
			vec3 position= getAtmosfereIntersection(cameraPosition, -sunLightVector);
			
			vec3 viewVector= normalize(position- cameraPosition);				
			
			float viewVectorStepSize= length(position- cameraPosition)/(NUM_STEPS- 1);
			float viewVectorRayleighOpticalDepth= 0.0;
			float viewVectorMieOpticalDepth= 0.0;
			vec3 viewPoint= cameraPosition;
			
			//Acumulate particles density along the light ray.
			for(int i= 0; i< NUM_STEPS; ++i){
				viewVectorRayleighOpticalDepth+= getAirDensity(viewPoint)*viewVectorStepSize;
				viewVectorMieOpticalDepth+= getAerosolDensity(viewPoint)*viewVectorStepSize;
				viewPoint+= viewVector*viewVectorStepSize;
			}
			
			//Calculate the optical depth coefficients.
			vec3 opticalDepthRayleigh= rayleighColorScatteringCoefficients*viewVectorRayleighOpticalDepth;
			vec3 opticalDepthMie= mieColorScatteringCoefficients*viewVectorMieOpticalDepth;
			
			//Calculate light extinction.
			vec3 extinction= exp(-opticalDepthRayleigh-opticalDepthMie);
			
			return sunColor*extinction;			
		}
	"
	
	semantics{
		rayleighParticlesConcentration= RAYLEIGH_PARTICLES_CONCENTRATION
		mieParticlesConcentration= MIE_PARTICLES_CONCENTRATION
		miePhaseFunctionG= MIE_PHASE_PARAMETER_G
		rayleighColorScatteringCoefficients= RAYLEIGH_COLOR_SCATTERING_COEFFICIENTS
		mieColorScatteringCoefficients= MIE_COLOR_SCATTERING_COEFFICIENTS
	}
}