shader{

	code= "

	//Pre-calculated values used for the attenuation.
	#define BORDER_START						0.5
	#define INVERSE_ONE_MINUS_BORDER_START		1.0/BORDER_START

	uniform sampler2D	propagationTexture0;
	uniform sampler2D	propagationTexture1;
	uniform sampler2D	propagationTexture2;
	uniform sampler2D	propagationTexture3;
	uniform vec3		lightPropagationVolumePosition;
	uniform float		lightPropagationVolumeWidth;
	uniform float		lightPropagationVolumeHeight;
	uniform float		lightPropagationVolumeLength;
	uniform float		propagationVolumeWidthResolution;
	uniform float		propagationVolumeHeightResolution;
	uniform float		propagationVolumeLengthResolution;
	
	vec3 sampleLightPropagationVolume(vec3 position, vec3 normal){
		
		//Calculate and clamp the texture coordinates.
		vec3 texCoord= (position- lightPropagationVolumePosition).xzy/vec3(lightPropagationVolumeWidth, lightPropagationVolumeLength, lightPropagationVolumeHeight)+ 0.5;

		//Calculate the attenuation near the border.
		//This is done by converting the texture coordinates from the [0, 1] range to the [-1, 1] range, then all
		//coordinates are converted to positive values and the largest coordinate is chosen. The distance of that
		//coordinate to the border start is measured and is used to perform a linear interpolation between the
		//border start and end.
		vec3 texCoord2= abs(texCoord*2.0 - 1.0);
		float largestCoordinate= max(texCoord2.x, max(texCoord2.y, texCoord2.z));
		float attenuation= 1.0- clamp((largestCoordinate- BORDER_START)*INVERSE_ONE_MINUS_BORDER_START, 0.0, 1.0);
		
		//Calculate the texture coordinates to address the unwrapped propagation texture.
		texCoord.z-= 1.0/propagationVolumeHeightResolution/2.0;
		float layer= floor(texCoord.z*propagationVolumeHeightResolution);
		texCoord.x= (texCoord.x+ layer)/propagationVolumeHeightResolution;
			
		//Read the lower layer.
		SH sh1;
		sh1.L00= texture2D(propagationTexture0, texCoord.xy).xyz;
		sh1.L1m1= texture2D(propagationTexture1, texCoord.xy).xyz;
		sh1.L10= texture2D(propagationTexture2, texCoord.xy).xyz;
		sh1.L11= texture2D(propagationTexture3, texCoord.xy).xyz;
			
		//Read the upper layer.
		texCoord.x+= 1.0/propagationVolumeHeightResolution;
		SH sh2;
		sh2.L00= texture2D(propagationTexture0, texCoord.xy).xyz;
		sh2.L1m1= texture2D(propagationTexture1, texCoord.xy).xyz;
		sh2.L10= texture2D(propagationTexture2, texCoord.xy).xyz;
		sh2.L11= texture2D(propagationTexture3, texCoord.xy).xyz;
			
		//Interpolate between the two layers.
		float weigth= fract(texCoord.z*propagationVolumeHeightResolution);
		sh1.L00= mix(sh1.L00, sh2.L00, weigth);
		sh1.L1m1= mix(sh1.L1m1, sh2.L1m1, weigth);
		sh1.L10= mix(sh1.L10, sh2.L10, weigth);
		sh1.L11= mix(sh1.L11, sh2.L11, weigth);
		
		//Calculate the illumination color from the Spherical Harmonic and apply the attenuation factor to it.
		//vec3 illuminationColor= sampleColorFromSH(sh1, -normal)*attenuation;

		//Sample the SH with a diffuse convolution.
		vec3 illuminationColor= 0.886227*sh1.L00*C1+ 2.0*0.511664*(max(-sh1.L1m1*normal.y, 0.0)+ max(-sh1.L10*normal.z, 0.0)+ max(-sh1.L11*normal.x, 0.0))*C2;
		
		return illuminationColor*attenuation;
	}
	"
	
	includes{
		file= resources/shaders/engine/lightPropagationVolumes/sphericalHarmonics.sh
	}
	
	semantics{
		propagationTexture0= LIGHT_PROPAGATION_ACCUMULATION_VOLUME_TEXTURE0
		propagationTexture1= LIGHT_PROPAGATION_ACCUMULATION_VOLUME_TEXTURE1
		propagationTexture2= LIGHT_PROPAGATION_ACCUMULATION_VOLUME_TEXTURE2
		propagationTexture3= LIGHT_PROPAGATION_ACCUMULATION_VOLUME_TEXTURE3
		lightPropagationVolumePosition= LIGHT_PROPAGATION_VOLUME_POSITION
		lightPropagationVolumeWidth= LIGHT_PROPAGATION_VOLUME_WIDTH
		lightPropagationVolumeHeight= LIGHT_PROPAGATION_VOLUME_HEIGHT
		lightPropagationVolumeLength= LIGHT_PROPAGATION_VOLUME_LENGTH
		propagationVolumeWidthResolution= LIGHT_PROPAGATION_VOLUME_WIDTH_RESOLUTION
		propagationVolumeHeightResolution= LIGHT_PROPAGATION_VOLUME_HEIGHT_RESOLUTION
		propagationVolumeLengthResolution= LIGHT_PROPAGATION_VOLUME_LENGTH_RESOLUTION
	}
}