shader{

	code= "
	uniform sampler2D	colorTexture;
	uniform sampler2D	normalTexture;
	uniform sampler2D	depthTexture;
	uniform mat4		invertedProjectionMatrix;
	uniform vec3		sunLightDirection;
	uniform float		propagationVolumeWorldSpaceTexelSize;
	uniform float		propagationVolumeLightsWeight;
	uniform vec3		lightPropagationVolumePosition;
	uniform float		lightPropagationVolumeWidth;
	uniform float		lightPropagationVolumeHeight;
	uniform float		lightPropagationVolumeLength;
	uniform float		propagationVolumeHeightResolution;
	
	varying vec3		lightColor;
	varying vec3		lightDirection;
	
	void main(){

		//Read the VPL color and normal from the RSM.
		vec3 color= texture2D(colorTexture, gl_MultiTexCoord0.st).xyz;
		vec3 normal= texture2D(normalTexture, gl_MultiTexCoord0.st).xyz;
		
		//Calculate the VPL position by reading the depth texture of the RSM and unprojecting from it.
		float depth= texture2D(depthTexture, gl_MultiTexCoord0.st).x*2.0- 1.0;
		vec4 position= vec4(gl_MultiTexCoord0.st*2.0- 1.0, depth, 1.0);
		position= invertedProjectionMatrix*position;		
		position= vec4(position.xyz/position.w, 1.0);
		
		//Half cell shifting of the position in the direction of normal and light to avoid self illumination.
		//Attention: actually, the length of the shifting is one cell not half to reduce self illumination.
		vec3 shiftDirection= normalize(normal+ -sunLightDirection);
		position.xyz+= shiftDirection*propagationVolumeWorldSpaceTexelSize;
		
		//Multiply the color by the light's weight.
		color*= propagationVolumeLightsWeight;



		//Injection=======================================================================================


		//Lay down the vertex on the unwrapped texture. This is done by finding to which layer the
		//vertex belongs to based on its Y coordinate and calculating an offset that is applied to
		//the vertex position.
		
		vec3 volumeDimensions= vec3(lightPropagationVolumeWidth, lightPropagationVolumeHeight, lightPropagationVolumeLength);
		vec3 positionInsideVolume= (position.xyz- (lightPropagationVolumePosition- volumeDimensions*0.5))/volumeDimensions;

		//Make sure the vertex is inside the layer, if not then
		//it will be ignored by assigning a black color to it.
		if(positionInsideVolume.x>= 0.0 && positionInsideVolume.x<= 1.0 && positionInsideVolume.y>= 0.0 && positionInsideVolume.y<= 1.0 && positionInsideVolume.z>= 0.0 && positionInsideVolume.z<= 1.0){
			float layer= floor(positionInsideVolume.y*propagationVolumeHeightResolution);
			position.x+= lightPropagationVolumeWidth*layer;
		}
		else
			color= vec3(0.0);

		gl_Position= gl_ModelViewProjectionMatrix*position;
		lightColor= color;
		lightDirection= normal;
	}
	"
	
	semantics{
		colorTexture= REFLECTIVE_SHADOW_MAP_COLOR_TEXTURE
		normalTexture= REFLECTIVE_SHADOW_MAP_NORMAL_TEXTURE
		depthTexture= REFLECTIVE_SHADOW_MAP_DEPTH_TEXTURE
		invertedProjectionMatrix= REFLECTIVE_SHADOW_MAP_INVERTED_MATRIX
		propagationVolumeWorldSpaceTexelSize= LIGHT_PROPAGATION_VOLUME_WORLDSPACE_TEXEL_SIZE
		sunLightDirection= SUN_LIGHT_DIRECTION
		propagationVolumeLightsWeight= LIGHT_PROPAGATION_VOLUME_LIGHTS_WEIGHT
		lightPropagationVolumePosition= LIGHT_PROPAGATION_VOLUME_POSITION
		lightPropagationVolumeWidth= LIGHT_PROPAGATION_VOLUME_WIDTH
		lightPropagationVolumeHeight= LIGHT_PROPAGATION_VOLUME_HEIGHT
		lightPropagationVolumeLength= LIGHT_PROPAGATION_VOLUME_LENGTH
		propagationVolumeHeightResolution= LIGHT_PROPAGATION_VOLUME_HEIGHT_RESOLUTION
	}
}