shader
{
	
	code= "
	#version 430

	uniform sampler2D	colorTexture;
	uniform sampler2D	circleOfConfusionTexture;
	uniform float		viewportWidth;
	uniform float		viewportHeight;

	in vec2				texCoord;

	struct BokehPoint
	{
		vec4	position;
		vec4	colorAndRadius;
	};

	layout (binding = 0, offset = 0) uniform atomic_uint	bokehPointCounter;

	layout (packed, binding = 1) buffer						BokehPointBuffer
	{
		writeonly BokehPoint bokehPoints[];
	};


	void main()
	{
		const float PI = 3.141592;
		const float MINIMUM_COC_RADIUS = 0.005;

		//Get the pixel's color.
		vec3 color = texture(colorTexture, texCoord).rgb;

		//Get the pixel's circle of confusion.
		float size = texture(circleOfConfusionTexture, texCoord).x;
		float radius = size / 2.0;				
		

		
		vec2 pixelSize = vec2(1.0 / viewportWidth, 1.0 / viewportHeight);
		vec3 averageColor = vec3(0, 0, 0);
		for(int y = -2; y <= 2; y++)
		{
			for(int x = -2; x <= 2; x++)
			{
				averageColor += texture(colorTexture, texCoord + vec2(x, y) * pixelSize).rgb;
			}
		}
		averageColor /= 25;
		float colorDifference = abs(length(averageColor) - length(color));


		//If the pixel is above the minimum luminance and circle of
		//confusion radius then add a new bokeh point to the buffer.
		if(radius > MINIMUM_COC_RADIUS && colorDifference > 1.0)
		{	
			//To keep the overall image brightness intact we have to decrease the intensity
			//of the bokeh proportionally to its size because we're distributing the energy
			//of the original pixel over a larger area. To do this we calculate the size of
			//the bokeh relative to the original pixel and square it to obtain the area where
			//the energy is distributed.
			float halfPixelSize = 1.0 / viewportHeight / 2.0;
			float scale = halfPixelSize / radius;
			scale = scale * scale;

			//We have scaled the energy assuming the bokeh has a square shape which is not true,
			//the true shape (e.g circle, hexagon, etc.) will fit inside that square but will not
			//cover all of its area thus resulting in discarded energy and overall lower brightness
			//of the image. We'll asume the shape is a unit circle that fits tightly inside a unit
			//square and scale the energy accordingly.
			float areaOfUnitCircle = PI * 0.5 * 0.5;
			float relativeSizeToUnitSquare = 1.0 / areaOfUnitCircle;
			scale *= relativeSizeToUnitSquare;

			//Add the point to the buffer.
			uint index = atomicCounterIncrement(bokehPointCounter);
			bokehPoints[index].position = vec4(texCoord, 0.0, 0.0);
			bokehPoints[index].colorAndRadius = vec4(color * scale, radius);

			//Replace the original pixel by a black one because its contribution
			//to the depth of field effect will be provided by the bokeh point
			gl_FragColor = vec4(0, 0, 0, 0);
		}
		else
		{
			//This pixel is not a candidate for a bokeh point so write it to the
			//frame buffer so it can be handled by the default depth of field blur.
			gl_FragColor = vec4(color, 1.0);
		}
	}
	"

	semantics
	{
		viewportWidth = VIEWPORT_WIDTH
		viewportHeight = VIEWPORT_HEIGHT
	}
}