shader{

	code= "
	const int												PIXEL_BLOCK_SIZE = 3;
	const float												NORMAL_TRESHOLD = 0.8;
	const float												POSITION_TRESHOLD = 0.5;
	const int												NUM_CONES = 16;

	uniform float											viewportWidth;
	uniform float											viewportHeight;
	uniform vec3											conesDirections[NUM_CONES];
	
	layout (rgba16f) writeonly uniform image2D 				frameBuffer;
	layout (local_size_x = 32, local_size_y = 32)			in;


	void main()
	{
		//Calculate the top left pixel coordinates and other useful data.
		const vec2 halfTexelOffset = vec2(0.5, 0.5);
		const vec2 viewportDimensions = vec2(viewportWidth, viewportHeight);
		uvec2 topLeftPixelCoord = gl_GlobalInvocationID.xy * PIXEL_BLOCK_SIZE;

		//Early out if we're processing outside the viewport limits.
		if(topLeftPixelCoord.x >= viewportWidth || topLeftPixelCoord.y >= viewportHeight)
		{
			return;
		}

		//Obtain the center pixel data.
		uvec2 centerPixelCoord = topLeftPixelCoord + uvec2(1, 1);
		vec2 centerPixelTexCoord = (vec2(centerPixelCoord) + halfTexelOffset) / viewportDimensions;

		vec3 centerPixelPosition = getPosition(centerPixelTexCoord);
		vec3 centerPixelGeometryNormal = getGeometryNormal(centerPixelTexCoord);
		
		//Trace each cone and store the results.
		vec3 cones[NUM_CONES];
		for(int i = 0; i < NUM_CONES; i++)
		{
			//Trace the cone and extract the resulting GI and occlusion values.
			vec4 coneResult = traceDiffuseCone(centerPixelPosition, centerPixelGeometryNormal, conesDirections[i]);
			vec3 giColor = coneResult.rgb;
			float occlusion = coneResult.a;

			//Calculate the sky lighting contribution.
			vec3 skyColor = sampleSkyMap(conesDirections[i], GI_CONE_ANGLE);
			vec3 skyLighting = skyColor * occlusion;
			
			//Calculate the cone color.
			cones[i] = giColor * DIFFUSE_GI_INTENSITY + skyLighting * SKYLIGHTING_INTENSITY;
		}


		//Calculate the lighting for each pixel in the block.
		for(int y = 0; y < PIXEL_BLOCK_SIZE; y++)
		{
			for(int x = 0; x < PIXEL_BLOCK_SIZE; x++)
			{
				//Get the pixel information.
				uvec2 pixelCoord = topLeftPixelCoord + uvec2(x, y);
				vec2 texCoord = (vec2(topLeftPixelCoord) + vec2(x, y) + halfTexelOffset) / viewportDimensions;
				vec3 color = getColor(texCoord);
				vec3 normal = getNormal(texCoord);

				//Go trough each cone and accumulate the lighting.
				vec3 lighting = vec3(0, 0, 0);
				for(int cone = 0; cone < NUM_CONES; cone++)
				{
					//Add the GI to the pixel color.
					float brdf = max(dot(normal, conesDirections[cone]), 0.0);
					lighting += cones[cone] * brdf;
				}

				//Normalize multiplying by the sphere surface area divided by the number of cones.
				lighting *= 4.0 * PI / NUM_CONES;

				//Apply the color.
				vec3 pixelColor = color * lighting;

				//Store the results.
				imageStore(frameBuffer, ivec2(pixelCoord), vec4(pixelColor, 1));
			}
		}
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
		file = resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
		file = resources/shaders/engine/voxelConeTracing/voxelConeTracing.sh
		file = resources/shaders/engine/atmosphere/skyLighting/skyMapRead.sh
	}

	semantics
	{
		viewportWidth = VIEWPORT_WIDTH
		viewportHeight = VIEWPORT_HEIGHT
	}
}