shader{

	code= "
	const int												PIXEL_BLOCK_SIZE = 3;
	const int												NUM_CONES = 16;

	uniform float											viewportWidth;
	uniform float											viewportHeight;
	uniform vec3											conesDirections[NUM_CONES];
	
	layout (local_size_x = 64)								in;
	
	layout (std140, binding = 0) buffer						DiffuseBlocksCounter
	{
		readonly uint diffuseBlocksCounter;
	};

	layout (std140, binding = 1) buffer						DiffuseBlocksBuffer
	{
		readonly uvec2 diffuseBlocks[];
	};

	layout (rgba16f) writeonly uniform image2D 				frameBuffer;


	void main()
	{
		//Get index of the block that we'll be processing.
		uint blockIndex = gl_GlobalInvocationID.x;

		//Make sure this thread is processing a valid pixel block.
		if(blockIndex < diffuseBlocksCounter)
		{
			//Fetch the block coordinates from the buffer.
			uvec2 topLeftPixelCoord = diffuseBlocks[blockIndex];


			for(int y = 0; y < PIXEL_BLOCK_SIZE; y++)
			{
				for(int x = 0; x < PIXEL_BLOCK_SIZE; x++)
				{
					//Calculate the pixel coordinates.
					const vec2 viewportDimensions = vec2(viewportWidth, viewportHeight);
					const vec2 halfTexelOffset = vec2(0.5, 0.5);		
					uvec2 pixelCoord = topLeftPixelCoord + uvec2(x, y);
					vec2 texCoord = (vec2(topLeftPixelCoord) + vec2(x, y) + halfTexelOffset) / viewportDimensions;

					//Get the pixel data.
					vec3 color = getColor(texCoord);
					vec3 position = getPosition(texCoord);
					vec3 normal = getNormal(texCoord);
					vec3 geometryNormal = getGeometryNormal(texCoord);

					vec3 lighting = vec3(0, 0, 0);

					//Trace each cone and store the results.
					for(int cone = 0; cone < NUM_CONES; cone++)
					{
						//Trace the cone and extract the resulting GI and occlusion values.
						vec4 coneResult = traceDiffuseCone(position, geometryNormal, conesDirections[cone]);
						vec3 giColor = coneResult.rgb;
						float occlusion = coneResult.a;

						//Calculate the sky lighting contribution.
						vec3 skyColor = sampleSkyMap(conesDirections[cone], GI_CONE_ANGLE);
						vec3 skyLighting = skyColor * occlusion;
			
						//Calculate the cone color.
						float brdf = max(dot(normal, conesDirections[cone]), 0.0);
						lighting += (giColor * DIFFUSE_GI_INTENSITY + skyLighting * SKYLIGHTING_INTENSITY) * brdf;
					}

					//Normalize multiplying by the sphere surface area divided by the number of cones.
					lighting *= 4.0 * PI / NUM_CONES;

					//Apply the color.
					vec3 pixelColor = color * lighting;

					//Store the results.
					imageStore(frameBuffer, ivec2(topLeftPixelCoord) + ivec2(x, y), vec4(pixelColor, 1));
				}
			}
		}
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
		file = resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
		file = resources/shaders/engine/voxelConeTracing/voxelConeTracing.sh
		file = resources/shaders/engine/atmosphere/skyLighting/skyMapRead.sh
	}

	semantics
	{
		viewportWidth = VIEWPORT_WIDTH
		viewportHeight = VIEWPORT_HEIGHT
	}
}