shader{

	code= "
	const int												PIXEL_BLOCK_SIZE = 3;
	const float												POSITION_THRESHOLD = 0.5;
	const float												NORMAL_THRESHOLD = 0.9;

	uniform float											viewportWidth;
	uniform float											viewportHeight;
	
	layout (local_size_x = 32, local_size_y = 32)			in;
	layout (binding = 0, offset = 0) uniform atomic_uint	diffuseBlocksCounter;
	
	layout (std140, binding = 1) buffer						DiffuseBlocksBuffer
	{
		writeonly uvec2 diffuseBlocks[];
	};


	void main()
	{
		//Calculate the top left pixel coordinates and other useful data.
		const vec2 halfTexelOffset = vec2(0.5, 0.5);
		const vec2 viewportDimensions = vec2(viewportWidth, viewportHeight);
		uvec2 topLeftPixelCoord = gl_GlobalInvocationID.xy * PIXEL_BLOCK_SIZE;

		//Early out if we're processing outside the viewport limits.
		if(topLeftPixelCoord.x >= viewportWidth || topLeftPixelCoord.y >= viewportHeight)
		{
			return;
		}

		//Obtain the center pixel data.
		uvec2 centerPixelCoord = topLeftPixelCoord + uvec2(1, 1);
		vec2 centerPixelTexCoord = (vec2(centerPixelCoord) + halfTexelOffset) / viewportDimensions;
		vec3 centerPixelPosition = getPosition(centerPixelTexCoord);

		//For if the pixels in the block are coherent.
		bool isDiffuseCoherent = true;
		bool isReflectionCoherent = true;

		for(int y = 0; y < PIXEL_BLOCK_SIZE; y++)
		{	
			for(int x = 0; x < PIXEL_BLOCK_SIZE; x++)
			{
				//Calculate the pixel coordinates.
				uvec2 pixelCoord = topLeftPixelCoord + uvec2(x, y);
				vec2 texCoord = (vec2(topLeftPixelCoord) + vec2(x, y) + halfTexelOffset) / viewportDimensions;

				//Get the pixel data.
				vec3 pixelPosition = getPosition(texCoord);

				//For a pixel block to be considered coherent for diffuse lighting all of its pixels must lay
				//within a small area in world space. Their normals are irrelevant because we'll be casting cones
				//in 360 degrees when processing the block so we'll have information about the lighting that comes
				//from all directions around that area.
				bool isPixelDiffuseCoherent = (distance(pixelPosition, centerPixelPosition) < POSITION_THRESHOLD);

				//Update the coeherency flag.
				isDiffuseCoherent = isDiffuseCoherent && isPixelDiffuseCoherent;
			}
		}

		//If the block is diffuse inchoerent then add it to the refinement buffer.
		if(isDiffuseCoherent == false)
		{
			uint blockID = atomicCounterIncrement(diffuseBlocksCounter);
			diffuseBlocks[blockID] = topLeftPixelCoord;
		}
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
		file = resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
	}

	semantics
	{
		viewportWidth = VIEWPORT_WIDTH
		viewportHeight = VIEWPORT_HEIGHT
	}
}