shader
{
	code= "
	#extension GL_EXT_shader_image_load_store : enable
	#extension GL_NV_shader_atomic_float : enable
	
	const int								NUM_CONES = 16;

	uniform sampler2D						colorTexture;
	uniform sampler2D						normalTexture;
	uniform mat4							objectMatrix2;
	uniform vec3							sunLightVector;
	uniform vec4							lightColor;
	uniform vec3							conesDirections[NUM_CONES];

	layout (r32f) coherent uniform image3D	voxelizationTextureNX;
	layout (r32f) coherent uniform image3D	voxelizationTexturePX;
	layout (r32f) coherent uniform image3D	voxelizationTextureNY;
	layout (r32f) coherent uniform image3D	voxelizationTexturePY;
	layout (r32f) coherent uniform image3D	voxelizationTextureNZ;
	layout (r32f) coherent uniform image3D	voxelizationTexturePZ;

	in vec3									position3;
	in vec3									normal3;
	in vec3									tangent3;
	in vec3									binormal3;
	in vec2									texCoord3;
	in vec3									positionInVolume;
	in vec2									projectedPosition;
	in vec2									clippingRectangleMin;
	in vec2									clippingRectangleMax;

	void main()
	{
		//Get the color.
		vec3 color = texture(colorTexture, texCoord3).rgb;

		//Calculate the world-space tangent matrix.
		mat3 objectMatrixNoTranslation = mat3(objectMatrix2);
		mat3 inverseTransposeObjectMatrix = transpose(inverse(objectMatrixNoTranslation));
		mat3 tangentSpaceMatrix = inverseTransposeObjectMatrix * mat3(tangent3, binormal3, normal3);

		//Get the normal.
		vec3 normal = texture2D(normalTexture, texCoord3).xyz * 2.0 - 1.0;
		vec3 worldSpaceNormal = normalize(normal * transpose(tangentSpaceMatrix));
		vec3 geometryNormal = normalize(tangentSpaceMatrix[2]);

		//Calculate the lighting.
		float shadow = sampleOverlappedShadowMap(position3, vec3(0), vec2(0));
		vec3 lighting = lightColor.rgb * max(dot(worldSpaceNormal, -sunLightVector), 0.0);
		
		
		//Calculate the resulting voxel color.
		vec3 voxelColor = color * lighting * shadow;


		//Calculate the indirect lighting by cone tracing the surroundings of this voxel.
		vec3 gi = vec3(0.0, 0.0, 0.0);
		for(unsigned int cone = 0; cone < NUM_CONES; cone++)
		{
			vec3 coneDirection = conesDirections[cone];
			vec3 coneResult = vec3(0.0, 0.0, 0.0);

			//Ignore this ray if it points behind the normal.
			if(dot(worldSpaceNormal, coneDirection) > 0)
			{
				//Trace the cone and extract the resulting GI and occlusion values.
				vec4 cone = traceDiffuseCone(position3, normal3, coneDirection);
				vec3 giColor = cone.rgb * DIFFUSE_GI_INTENSITY;
				float occlusion = cone.a;

				//Calculate the sky lighting contribution.
				vec3 skyColor = sampleSkyMap(coneDirection, GI_CONE_ANGLE);
				vec3 skyLighting = skyColor * occlusion * SKYLIGHTING_INTENSITY;

				//Calculate the final color.
				coneResult = color * (giColor + skyLighting) * dot(worldSpaceNormal, coneDirection);
			}

			//Normalize multiplying by the sphere surface area divided by the number of cones.
			coneResult *= 4.0 * PI / NUM_CONES;

			voxelColor += coneResult;
		}

		//Write to the voxel volume only if the pixel is inside the clipping rectangle.
		bool insideClippingRectangle = (projectedPosition.x >= clippingRectangleMin.x && projectedPosition.y >= clippingRectangleMin.y && projectedPosition.x <= clippingRectangleMax.x && projectedPosition.y <= clippingRectangleMax.y);

		if(insideClippingRectangle)
		{
			ivec3 pixelCoords = ivec3(positionInVolume * voxelVolumeResolution);
			
			//Scale the Z coordinates by 4 to take into account the layout of RGB channels and fragment counter in the texture.
			pixelCoords.z *= 4;

			//Negative X.
			if(geometryNormal.x < 0.0)
			{
				vec3 weightedColor = voxelColor * max(-geometryNormal.x, 0.0);
				imageAtomicAdd(voxelizationTextureNX, pixelCoords + ivec3(0, 0, 0), weightedColor.r);
				imageAtomicAdd(voxelizationTextureNX, pixelCoords + ivec3(0, 0, 1), weightedColor.g);
				imageAtomicAdd(voxelizationTextureNX, pixelCoords + ivec3(0, 0, 2), weightedColor.b);
				imageAtomicAdd(voxelizationTextureNX, pixelCoords + ivec3(0, 0, 3), 1.0);
			}

			//Positive X.
			if(geometryNormal.x > 0.0)
			{
				vec3 weightedColor = voxelColor * max(geometryNormal.x, 0.0);
				imageAtomicAdd(voxelizationTexturePX, pixelCoords + ivec3(0, 0, 0), weightedColor.r);
				imageAtomicAdd(voxelizationTexturePX, pixelCoords + ivec3(0, 0, 1), weightedColor.g);
				imageAtomicAdd(voxelizationTexturePX, pixelCoords + ivec3(0, 0, 2), weightedColor.b);
				imageAtomicAdd(voxelizationTexturePX, pixelCoords + ivec3(0, 0, 3), 1.0);
			}

			//Negative Y.
			if(geometryNormal.y < 0.0)
			{
				vec3 weightedColor = voxelColor * max(-geometryNormal.y, 0.0);
				imageAtomicAdd(voxelizationTextureNY, pixelCoords + ivec3(0, 0, 0), weightedColor.r);
				imageAtomicAdd(voxelizationTextureNY, pixelCoords + ivec3(0, 0, 1), weightedColor.g);
				imageAtomicAdd(voxelizationTextureNY, pixelCoords + ivec3(0, 0, 2), weightedColor.b);
				imageAtomicAdd(voxelizationTextureNY, pixelCoords + ivec3(0, 0, 3), 1.0);
			}

			//Positive Y.
			if(geometryNormal.y > 0.0)
			{
				vec3 weightedColor = voxelColor * max(geometryNormal.y, 0.0);
				imageAtomicAdd(voxelizationTexturePY, pixelCoords + ivec3(0, 0, 0), weightedColor.r);
				imageAtomicAdd(voxelizationTexturePY, pixelCoords + ivec3(0, 0, 1), weightedColor.g);
				imageAtomicAdd(voxelizationTexturePY, pixelCoords + ivec3(0, 0, 2), weightedColor.b);
				imageAtomicAdd(voxelizationTexturePY, pixelCoords + ivec3(0, 0, 3), 1.0);
			}

			//Negative Z.
			if(geometryNormal.z < 0.0)
			{
				vec3 weightedColor = voxelColor * max(-geometryNormal.z, 0.0);
				imageAtomicAdd(voxelizationTextureNZ, pixelCoords + ivec3(0, 0, 0), weightedColor.r);
				imageAtomicAdd(voxelizationTextureNZ, pixelCoords + ivec3(0, 0, 1), weightedColor.g);
				imageAtomicAdd(voxelizationTextureNZ, pixelCoords + ivec3(0, 0, 2), weightedColor.b);
				imageAtomicAdd(voxelizationTextureNZ, pixelCoords + ivec3(0, 0, 3), 1.0);
			}

			//Positive Z.
			if(geometryNormal.z > 0.0)
			{
				vec3 weightedColor = voxelColor * max(geometryNormal.z, 0.0);
				imageAtomicAdd(voxelizationTexturePZ, pixelCoords + ivec3(0, 0, 0), weightedColor.r);
				imageAtomicAdd(voxelizationTexturePZ, pixelCoords + ivec3(0, 0, 1), weightedColor.g);
				imageAtomicAdd(voxelizationTexturePZ, pixelCoords + ivec3(0, 0, 2), weightedColor.b);
				imageAtomicAdd(voxelizationTexturePZ, pixelCoords + ivec3(0, 0, 3), 1.0);
			}
		}
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
		file = resources/shaders/engine/shadows/overlappedShadowMapping.sh
		file = resources/shaders/engine/voxelConeTracing/voxelConeTracing.sh
		file = resources/shaders/engine/atmosphere/skyLighting/skyMapRead.sh
	}

	semantics
	{
		colorTexture = MATERIAL_COLOR_TEXTURE
		normalTexture = MATERIAL_NORMAL_TEXTURE
		objectMatrix2 = OBJECT_TRANSFORM_MATRIX
		sunLightVector = SUN_LIGHT_DIRECTION
		lightColor = LIGHT_COLOR
		voxelVolumeResolution = VOXEL_VOLUME_RESOLUTION
	}
}